@@ -10,9 +10,15 @@
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
+#ifdef HAVE_NET_SOCKET_H
+#include <net/socket.h>
+#else
 #include <sys/socket.h>
+#endif
 #include <netinet/in.h>
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
 #include <netdb.h>
 #include "i18n.h"
 #include "mx.h"
@@ -117,8 +123,6 @@ int is_host_alias(const char *name, struct query *ctl)
 	return(TRUE);
     else if (str_in_list(&lead_server->akalist, name, TRUE))
 	return(TRUE);
-    else if (!ctl->server.dns)
-	return(FALSE);
 
     /*
      * Now check for a suffix match on the akalist.  The theory here is
@@ -131,18 +135,20 @@ int is_host_alias(const char *name, struct query *ctl)
 	char	*ep;
 
 	/*
-	 * Test is <= here because str_in_list() should have caught the
+	 * Test is >= here because str_in_list() should have caught the
 	 * equal-length case above.  Doing it this way guarantees that
 	 * ep[-1] is a valid reference.
 	 */
-	if (strlen(idl->id) <= namelen)
+	if (strlen(idl->id) >= namelen)
 	    continue;
-	ep = idl->id + (strlen(idl->id) - namelen);
+	ep = (char *)name + (namelen - strlen(idl->id));
 	/* a suffix led by . must match */
-	if (ep[-1] == '.' && !strcmp(ep, name))
+	if (ep[-1] == '.' && !strcmp(ep, idl->id))
 	    return(TRUE);
     }
 
+    if (!ctl->server.dns)
+	return(FALSE);
 #ifndef HAVE_RES_SEARCH
     return(FALSE);
 #else
@@ -180,9 +186,10 @@ int is_host_alias(const char *name, struct query *ctl)
 	switch (h_errno)
 	{
 	case HOST_NOT_FOUND:	/* specified host is unknown */
+#ifndef __BEOS__
 	case NO_ADDRESS:	/* valid, but does not have an IP address */
 	    break;
-
+#endif
 	case NO_RECOVERY:	/* non-recoverable name server error */
 	case TRY_AGAIN:		/* temporary error on authoritative server */
 	default:
@@ -206,10 +213,11 @@ int is_host_alias(const char *name, struct query *ctl)
 	switch (h_errno)
 	{
 	case HOST_NOT_FOUND:	/* specified host is unknown */
+#ifndef __BEOS__
 	case NO_ADDRESS:	/* valid, but does not have an IP address */
 	    return(FALSE);
 	    break;
-
+#endif
 	case NO_RECOVERY:	/* non-recoverable name server error */
 	case TRY_AGAIN:		/* temporary error on authoritative server */
 	default:
