@@ -922,7 +922,9 @@ static int readheaders(int sock,
 	else if (from_offs >= 0 && (ap = nxtaddr(msgblk.headers + from_offs)));
 	else if (reply_to_offs >= 0 && (ap = nxtaddr(msgblk.headers + reply_to_offs)));
 	else if (app_from_offs >= 0 && (ap = nxtaddr(msgblk.headers + app_from_offs)));
-	if (ap) strcpy( msgblk.return_path, ap );
+	/* multi-line MAIL FROM addresses confuse SMTP terribly */
+	if (ap && !strchr(ap, '\n')) 
+	    strcpy(msgblk.return_path, ap);
     }
 
     /* cons up a list of local recipients */
@@ -1065,15 +1067,18 @@ static int readheaders(int sock,
 	    /*
 	     * This header is technically invalid under RFC822.
 	     * POP3, IMAP, etc. are not legal mail-parameter values.
-	     *
-	     * We used to include ctl->remotename in this log line,
-	     * but this can be secure information that would be bad
-	     * to reveal.
 	     */
-	    sprintf(buf, "\tby %s with %s (fetchmail-%s)\r\n",
+	    sprintf(buf, "\tby %s with %s (fetchmail-%s)",
 		    fetchmailhost,
 		    protocol->name,
 		    VERSION);
+	    if (ctl->tracepolls)
+	    {
+		sprintf(buf + strlen(buf), " polling %s account %s",
+			ctl->server.pollname, 
+			ctl->remotename);
+	    }
+	    strcat(buf, "\r\n");
 	    n = stuffline(ctl, buf);
 	    if (n != -1)
 	    {
@@ -1823,7 +1828,7 @@ const int maxfetch;		/* maximum number of messages to fetch */
 		    report(stderr, _("Authorization failure on %s@%s%s\n"), 
 			   ctl->remotename,
 			   ctl->server.truename,
-			   (ctl->wehaveauthed ? " (previously authorized)" : " ")
+			   (ctl->wehaveauthed ? _(" (previously authorized)") : "")
 			);
 
 		    /*
@@ -2552,6 +2557,8 @@ const struct method *proto;	/* protocol method table */
 	int fetchlimit = NUM_VALUE_OUT(ctl->fetchlimit);
 
 	do {
+	    if (fetchlimit > 0 && (expunge == 0 || expunge > fetchlimit - totalcount))
+		expunge = fetchlimit - totalcount;
 	    ok = do_session(ctl, proto, expunge);
 	    totalcount += expunge;
 	    if (NUM_SPECIFIED(ctl->fetchlimit) && totalcount >= fetchlimit)
