@@ -849,6 +849,7 @@ static void optmerge(struct query *h2, struct query *h1, int force)
 #endif
     FLAG_MERGE(expunge);
 
+    FLAG_MERGE(tracepolls);
     FLAG_MERGE(properties);
 #undef FLAG_MERGE
 }
@@ -1111,7 +1112,7 @@ static int load_params(int argc, char **argv, int optind)
 		ctl->server.truename = xstrdup(leadname);
 	    }
 #ifdef HAVE_GETHOSTBYNAME
-	    else if (ctl->active && !configdump)
+	    else if (ctl->active && ctl->server.dns && !configdump)
 	    {
 		if (ctl->server.authenticate==A_KERBEROS_V4 ||
 		      ctl->server.authenticate==A_KERBEROS_V5 ||
@@ -1376,7 +1377,7 @@ static int query_host(struct query *ctl)
 	for (i = 0; i < sizeof(autoprobe)/sizeof(autoprobe[0]); i++)
 	{
 	    ctl->server.protocol = autoprobe[i];
-	    if ((st = query_host(ctl)) == PS_SUCCESS || st == PS_NOMAIL || st == PS_AUTHFAIL || st == PS_LOCKBUSY || st == PS_SMTP)
+	    if ((st = query_host(ctl)) == PS_SUCCESS || st == PS_NOMAIL || st == PS_AUTHFAIL || st == PS_LOCKBUSY || st == PS_SMTP || st == PS_MAXFETCH)
 		break;
 	}
 	ctl->server.protocol = P_AUTO;
@@ -1817,6 +1818,11 @@ static void dump_params (struct runctl *runp,
 	    }
 	}
 
+        if (ctl->tracepolls)
+            printf(_("  Poll trace information will be added to the Received header.\n"));
+        else if (outlevel >= O_VERBOSE)
+            printf(_("  No poll trace information will be added to the Received header.\n.\n"));
+
 	if (ctl->properties)
 	    printf(_("  Pass-through properties \"%s\".\n"),
 		   visbuf(ctl->properties));
