@@ -41,7 +41,7 @@ static char lastok[POPBUFSIZE+1];
 
 #define DOTLINE(s)	(s[0] == '.' && (s[1]=='\r'||s[1]=='\n'||s[1]=='\0'))
 
-int pop3_ok (int sock, char *argbuf)
+static int pop3_ok (int sock, char *argbuf)
 /* parse command response */
 {
     int ok;
@@ -49,8 +49,7 @@ int pop3_ok (int sock, char *argbuf)
     char *bufp;
 
     if ((ok = gen_recv(sock, buf, sizeof(buf))) == 0)
-    {
-	bufp = buf;
+    {	bufp = buf;
 	if (*bufp == '+' || *bufp == '-')
 	    bufp++;
 	else
@@ -99,6 +98,10 @@ int pop3_ok (int sock, char *argbuf)
 		     /* these are blessed by RFC 2449 */
 		     || strstr(bufp,"[IN-USE]")||strstr(bufp,"[LOGIN-DELAY]"))
 		ok = PS_LOCKBUSY;
+	    else if ((strstr(bufp,"Service")
+		     || strstr(bufp,"service"))
+			 && (strstr(bufp,"unavailable")))
+		ok = PS_SERVBUSY;
 	    else
 		ok = PS_AUTHFAIL;
 	    /*
@@ -119,7 +122,7 @@ int pop3_ok (int sock, char *argbuf)
     return(ok);
 }
 
-int pop3_getauth(int sock, struct query *ctl, char *greeting)
+static int pop3_getauth(int sock, struct query *ctl, char *greeting)
 /* apply for connection authorization */
 {
     int ok;
@@ -128,6 +131,16 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 #if OPIE_ENABLE
     char *challenge;
 #endif /* OPIE_ENABLE */
+#if defined(GSSAPI)
+    flag has_gssapi = FALSE;
+#endif /* defined(GSSAPI) */
+#if defined(KERBEROS_V4) || defined(KERBEROS_V5)
+    flag has_kerberos = FALSE;
+#endif /* defined(KERBEROS_V4) || defined(KERBEROS_V5) */
+    flag has_cram = FALSE;
+#ifdef OPIE_ENABLE
+    flag has_otp = FALSE;
+#endif /* OPIE_ENABLE */
 
 #ifdef SDPS_ENABLE
     /*
@@ -187,16 +200,6 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	    && strchr(greeting, '<') 
 	    && gen_transact(sock, "CAPA") == 0)
 	{
-#if defined(GSSAPI)
-	    flag has_gssapi = FALSE;
-#endif /* defined(GSSAPI) */
-#if defined(KERBEROS_V4) || defined(KERBEROS_V5)
-	    flag has_kerberos = FALSE;
-#endif /* defined(KERBEROS_V4) || defined(KERBEROS_V5) */
-	    flag has_cram = FALSE;
-#ifdef OPIE_ENABLE
-	    flag has_opie = FALSE;
-#endif /* OPIE_ENABLE */
 	    char buffer[64];
 
 	    /* determine what authentication methods we have available */
@@ -208,10 +211,10 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 		if (strstr(buffer, "GSSAPI"))
 		    has_gssapi = TRUE;
 #endif /* defined(GSSAPI) */
-#if defined(KERBEROS_V4) || defined(KERBEROS_V5)
+#if defined(KERBEROS_V4)
 		if (strstr(buffer, "KERBEROS_V4"))
 		    has_kerberos = TRUE;
-#endif /* defined(KERBEROS_V4) || defined(KERBEROS_V5) */
+#endif /* defined(KERBEROS_V4)  */
 #ifdef OPIE_ENABLE
 		if (strstr(buffer, "X-OTP"))
 		    has_otp = TRUE;
@@ -219,50 +222,68 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 		if (strstr(buffer, "CRAM-MD5"))
 		    has_cram = TRUE;
 	    }
-
-	    /*
-	     * Here's where we set priorities.  Note that we must do tests
-	     * in *reverse* order of desirability.
-	     */
-	    if (has_cram)
-		ctl->server.authenticate = A_CRAM_MD5;
-#ifdef OPIE_ENABLE
-	    if (has_opie)
-		ctl->server.authenticate = A_OTP;
-#endif /* OPIE_ENABLE */
-#if defined(GSSAPI)
-	    if (has_gssapi)
-		ctl->server.authenticate = A_GSSAPI;
-#endif /* defined(GSSAPI) */
-#if defined(KERBEROS_V4) || defined(KERBEROS_V5)
-	    if (has_kerberos)
-		ctl->server.authenticate = A_KERBEROS_V4;
-#endif /* defined(KERBEROS_V4) || defined(KERBEROS_V5) */
 	}
 
 	/*
 	 * OK, we have an authentication type now.
 	 */
-#if defined(KERBEROS_V4) || defined(KERBEROS_V5)
-	if (ctl->server.authenticate == A_KERBEROS_V4
-	    	|| ctl->server.authenticate == A_KERBEROS_V5)
-	    return(do_rfc1731(sock, "AUTH", ctl->server.truename));
+#if defined(KERBEROS_V4)
+	/* 
+	 * Servers doing KPOP have to go through a dummy login sequence
+	 * rather than doing SASL.
+	 */
+	if (has_kerberos &&
+#if INET6_ENABLE
+	    ctl->server.service && (strcmp(ctl->server.service, KPOP_PORT)!=0)
+#else /* INET6_ENABLE */
+	    ctl->server.port != KPOP_PORT
+#endif /* INET6_ENABLE */
+	    && (ctl->server.authenticate == A_KERBEROS_V4
+	     || ctl->server.authenticate == A_KERBEROS_V5
+	     || ctl->server.authenticate == A_ANY))
+	{
+	    ok = do_rfc1731(sock, "AUTH", ctl->server.truename);
+	    if (ok == PS_SUCCESS || ctl->server.authenticate != A_ANY)
+		break;
+	}
 #endif /* defined(KERBEROS_V4) || defined(KERBEROS_V5) */
+
 #if defined(GSSAPI)
-	if (ctl->server.authenticate==A_GSSAPI))
-	    return(do_gssauth(sock, "AUTH", 
-			      ctl->server.truename, ctl->remotename));
+	if (has_gssapi &&
+	    (ctl->server.authenticate == A_GSSAPI ||
+	     ctl->server.authenticate == A_ANY))
+	{
+	    ok = do_gssauth(sock,"AUTH",ctl->server.truename,ctl->remotename);
+	    if (ok == PS_SUCCESS || ctl->server.authenticate != A_ANY)
+		break;
+	}
 #endif /* defined(GSSAPI) */
+
 #ifdef OPIE_ENABLE
-	if (ctl->server.authenticate == A_OTP)
-	    do_otp(sock, "AUTH", ctl)
+	if (has_otp &&
+	    (ctl->server.authenticate == A_OTP ||
+	     ctl->server.authenticate == A_ANY))
+	{
+	    ok = do_otp(sock, "AUTH", ctl);
+	    if (ok == PS_SUCCESS || ctl->server.authenticate != A_ANY)
+		break;
+	}
 #endif /* OPIE_ENABLE */
-	if (ctl->server.authenticate == A_CRAM_MD5)
-	    return(do_cram_md5(sock, "AUTH", ctl));
+
+	if (has_cram &&
+	    (ctl->server.authenticate == A_CRAM_MD5 ||
+	     ctl->server.authenticate == A_ANY))
+	{
+	    ok = do_cram_md5(sock, "AUTH", ctl);
+	    if (ok == PS_SUCCESS || ctl->server.authenticate != A_ANY)
+		break;
+	}
 
 	/* ordinary validation, no one-time password or RPA */ 
 	gen_transact(sock, "USER %s", ctl->remotename);
+	strcpy(shroud, ctl->password);
 	ok = gen_transact(sock, "PASS %s", ctl->password);
+	shroud[0] = '\0';
 	break;
 
     case P_APOP:
@@ -332,8 +353,7 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
     return(PS_SUCCESS);
 }
 
-static int
-pop3_gettopid( int sock, int num , char *id)
+static int pop3_gettopid( int sock, int num , char *id)
 {
     int ok;
     int got_it;
@@ -356,8 +376,7 @@ pop3_gettopid( int sock, int num , char *id)
     return 0;
 }
 
-static int
-pop3_slowuidl( int sock,  struct query *ctl, int *countp, int *newp)
+static int pop3_slowuidl( int sock,  struct query *ctl, int *countp, int *newp)
 {
     /* This approach tries to get the message headers from the
      * remote hosts and compares the message-id to the already known
@@ -718,7 +737,6 @@ const static struct method pop3 =
     FALSE,		/* this is not a tagged protocol */
     TRUE,		/* this uses a message delimiter */
     pop3_ok,		/* parse command response */
-    NULL,		/* no password canonicalization */
     pop3_getauth,	/* get authorization */
     pop3_getrange,	/* query range of messages */
     pop3_getsizes,	/* we can get a list of sizes */
